#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "orb/orbit.h"
#include "echo.h"

#define MULTI_THREAD
int             debug2;
int             niters = 1;
long long       loop = 1000000000L;

#ifdef MULTI_THREAD
#define MAXTHREADS      20
GThread        *threads[MAXTHREADS];
int             nthreads = 1;
#endif

Echo            echo_client;

void
do_Echo_echoNum ()
{
      CORBA_Environment ev;
      CORBA_long      retval;
      int             i;

      CORBA_exception_init (&ev);

      for (i = 0; i < niters; i++) {
	    retval = Echo_echoNum (echo_client, 10, &ev);

	    if (ev._major != CORBA_NO_EXCEPTION) {
		  printf ("we got exception %d from echoNum!\n", ev._major);
		  exit (-1);
	    }
	    g_print ("TID = %d : %d,   retval = %d ...\n", pthread_self(), i, retval);
      }
}

int
main (int argc, char *argv[])
{
      CORBA_Environment ev;
      CORBA_ORB       orb;

      FILE           *ifp;
      char            c, *ior;
      char            filebuffer[1024];
      char            retstr[1024];
      CORBA_long      l, retval;
      int             i, j;

      CORBA_exception_init (&ev);
      orb = CORBA_ORB_init (&argc, argv, "orbit-local-mt-orb", &ev);

      while ((c = getopt (argc, argv, "Hhdi:l:t:")) != -1) {
	    switch (c) {
	       case 'i':
		     niters = atoi (optarg);
		     break;
	       case 'd':
		     debug2 = 1;
		     break;
	       case 'l':
		     loop = atoi (optarg);
		     break;
	       case 't':
		     nthreads = atoi (optarg);
		     if (nthreads >= 21) {
			   printf (" too many threads(%d > 20) ...\n",
				 nthreads);
			   exit (1);
		     }
		     break;
	       case 'H':
	       case 'h':
	       default:
		     printf (" Options : Hhdi:p:t:l: \n");
		     exit (0);
	    }
      }

      ifp = fopen ("echo.ior", "r");
      if (ifp == NULL) {
	    g_error ("No echo.ior file!");
	    exit (-1);
      }

      fgets (filebuffer, 1024, ifp);
      ior = g_strdup (filebuffer);

      fclose (ifp);

      echo_client = CORBA_ORB_string_to_object (orb, ior, &ev);
      if (!echo_client) {
	    printf ("Cannot bind to %s\n", ior);
	    return 1;
      }

#ifdef MULTI_THREAD
      for (j = 0; j < loop; j++) {
	    for (i = 0; i < nthreads; i++) {
		  threads[i] = g_thread_create (do_Echo_echoNum,
			echo_client, 0, TRUE, TRUE, G_THREAD_PRIORITY_NORMAL);
	    }

	    for (i = 0; i < nthreads; i++) {
		  g_thread_join (threads[i]);
	    }
      }

#else
      for (j = 0; j < loop; j++) {
	      do_Echo_echoNum ();
      }
#endif

      /* Clean up */
      CORBA_Object_release (echo_client, &ev);
      CORBA_Object_release ((CORBA_Object) orb, &ev);

      return 0;
}
