#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "orb/orbit.h"
#include "echo.h"

Echo echo_client = CORBA_OBJECT_NIL;

/*** App-specific servant structures ***/
typedef struct
{
   POA_Echo servant;
   PortableServer_POA poa;

}
impl_POA_Echo;

/*** Implementation stub prototypes ***/
static void impl_Echo__destroy(impl_POA_Echo * servant,

			       CORBA_Environment * ev);
static CORBA_long
impl_Echo_echoNum(impl_POA_Echo * servant,
		  CORBA_long input, CORBA_Environment * ev);

/*** epv structures ***/
static PortableServer_ServantBase__epv impl_Echo_base_epv = {
   NULL,			/* _private data */
   NULL,			/* finalize routine */
   NULL,			/* default_POA routine */
};
static POA_Echo__epv impl_Echo_epv = {
   NULL,			/* _private */
   (gpointer) & impl_Echo_echoNum,

};

/*** vepv structures ***/
static POA_Echo__vepv impl_Echo_vepv = {
   &impl_Echo_base_epv,
   &impl_Echo_epv,
};

static POA_Echo poa_echo_servant = { NULL, &impl_Echo_vepv };

/*** Stub implementations ***/
static Echo
impl_Echo__create(PortableServer_POA poa, CORBA_Environment * ev)
{
   Echo retval;
   impl_POA_Echo *newservant;
   PortableServer_ObjectId *objid;

   newservant = g_new0(impl_POA_Echo, 1);
   newservant->servant.vepv = &impl_Echo_vepv;
   newservant->poa = poa;
   POA_Echo__init((PortableServer_Servant) newservant, ev);
   objid = PortableServer_POA_activate_object(poa, newservant, ev);
   CORBA_free(objid);
   retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

   return retval;
}

static void
impl_Echo__destroy(impl_POA_Echo * servant, CORBA_Environment * ev)
{
   PortableServer_ObjectId *objid;

   objid = PortableServer_POA_servant_to_id(servant->poa, servant, ev);
   PortableServer_POA_deactivate_object(servant->poa, objid, ev);
   CORBA_free(objid);

   POA_Echo__fini((PortableServer_Servant) servant, ev);
   g_free(servant);
}

int
main (int argc, char *argv[])
{
    PortableServer_ObjectId objid = {0, sizeof("myEchoString"), "myEchoString"};
    PortableServer_POA poa;

    CORBA_Environment ev;
    char *retval;
    CORBA_ORB orb;
    FILE * ofp;

    signal(SIGINT, exit);
    signal(SIGTERM, exit);

    CORBA_exception_init(&ev);

	/* init the ORB (AI) */
    orb = CORBA_ORB_init(&argc, argv, "orbit-local-mt-orb", &ev);



	/* get our internal reference to the object. This is cool because
	 * this reference is handled the same way in the server and client (AI)*/

#define NEW_ORBIT
#ifdef NEW_ORBIT
   POA_Echo__init(&poa_echo_servant, &ev);

    /* POA init stuff */
    poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
    PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(poa, &ev), &ev);
    PortableServer_POA_activate_object_with_id(poa,
					       &objid, &poa_echo_servant, &ev);

   echo_client = impl_Echo__create(poa, &ev);
#else
	/* POA init stuff */
    poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
    PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(poa, &ev), &ev);
    PortableServer_POA_activate_object_with_id(poa,
					       &objid, &poa_echo_servant, &ev);

   /* init our POA servant */
    POA_Echo__init(&poa_echo_servant, &ev);
    echo_client = PortableServer_POA_servant_to_reference(poa,
							  &poa_echo_servant,
							  &ev);
#endif

	
    if (!echo_client) {
        printf("Cannot get objref\n");
        return 1;
    }

	/* make an IOR string (serialize the object ref) so that clients
	* can use it to call us. This is really amazing because this object
	* ref contains the host-name, orb-port, and all the other interface
	* information so this object can be used from anything connected to
	* the network. Thank god we are using Linux! */
    retval = CORBA_ORB_object_to_string(orb, echo_client, &ev);

    ofp = fopen("echo.ior","w");

    fprintf(ofp,"%s", retval);
    fclose(ofp);

    CORBA_free(retval);

    fprintf(stdout,"Written the file echo.ior with the IOR of this server.\n Now waiting for requests...\n");
    fflush(stdout);

	/* ORB Run (AI)*/
    CORBA_ORB_run(orb, &ev);

    return 0;
}

static CORBA_long
impl_Echo_echoNum(impl_POA_Echo * servant,
		  CORBA_long input, CORBA_Environment * ev)
{
   return input + 100;
}
